# Phase planes for the model:
# Author: Robert W. Buchkowski
# Created: Feb 16/2021

# This runs as a sub-routine of the script 16_ODE_equilibrium_analysis.R
# The script won't run without the first bit of the above script.

# Create the 6 functions necessary ----

submod_BxxS <- function(t, y, pars){
  with(as.list(c(pars,y)),{
    # Equation for adult fir:
    dB = lambda*J + rB*B - rB*B*B/KB - aBS*B*S - hb*B
    
    # Equation for juvenile fir:
    dJ = pb + gB*B - aJ*J - hsb*cMJ*M*J - lambda*J - aJS*J*S
    
    # Equation for moose:
    dM = rM*M - beta*rM*M*M/KM + hsb*eM*cMJ*M*J - hm*M
    
    # Equation for adult spruce
    dS = rS*S - rS*S*S/KS - aSB*S*B + ps - hs*S
    
    return(list(c(dB, dS)))
    
  })
}

submod_xJMx <- function(t, y, pars){
  with(as.list(c(pars,y)),{
    # Equation for adult fir:
    dB = lambda*J + rB*B - rB*B*B/KB - aBS*B*S - hb*B
    
    # Equation for juvenile fir:
    dJ = pb + gB*B - aJ*J - hsb*cMJ*M*J - lambda*J - aJS*J*S
    
    # Equation for moose:
    dM = rM*M - beta*rM*M*M/KM + hsb*eM*cMJ*M*J - hm*M
    
    # Equation for adult spruce
    dS = rS*S - rS*S*S/KS - aSB*S*B + ps - hs*S
    return(list(c(dJ, dM)))
    
  })
}

submod_BJxx <- function(t, y, pars){
  with(as.list(c(pars,y)),{
    # Equation for adult fir:
    dB = lambda*J + rB*B - rB*B*B/KB - aBS*B*S - hb*B
    
    # Equation for juvenile fir:
    dJ = pb + gB*B - aJ*J - hsb*cMJ*M*J - lambda*J - aJS*J*S
    
    # Equation for moose:
    dM = rM*M - beta*rM*M*M/KM + hsb*eM*cMJ*M*J - hm*M
    
    # Equation for adult spruce
    dS = rS*S - rS*S*S/KS - aSB*S*B + ps - hs*S
    return(list(c(dB, dJ)))
    
  })
}

submod_xxMS <- function(t, y, pars){
  with(as.list(c(pars,y)),{
    # Equation for adult fir:
    dB = lambda*J + rB*B - rB*B*B/KB - aBS*B*S - hb*B
    
    # Equation for juvenile fir:
    dJ = pb + gB*B - aJ*J - hsb*cMJ*M*J - lambda*J - aJS*J*S
    
    # Equation for moose:
    dM = rM*M - beta*rM*M*M/KM + hsb*eM*cMJ*M*J - hm*M
    
    # Equation for adult spruce
    dS = rS*S - rS*S*S/KS - aSB*S*B + ps - hs*S
    return(list(c(dM, dS)))
    
  })
}

submod_xJxS <- function(t, y, pars){
  with(as.list(c(pars,y)),{
    # Equation for adult fir:
    dB = lambda*J + rB*B - rB*B*B/KB - aBS*B*S - hb*B
    
    # Equation for juvenile fir:
    dJ = pb + gB*B - aJ*J - hsb*cMJ*M*J - lambda*J - aJS*J*S
    
    # Equation for moose:
    dM = rM*M - beta*rM*M*M/KM + hsb*eM*cMJ*M*J - hm*M
    
    # Equation for adult spruce
    dS = rS*S - rS*S*S/KS - aSB*S*B + ps - hs*S
    return(list(c(dJ,dS)))
    
  })
}

submod_BxMx <- function(t, y, pars){
  with(as.list(c(pars,y)),{
    # Equation for adult fir:
    dB = lambda*J + rB*B - rB*B*B/KB - aBS*B*S - hb*B
    
    # Equation for juvenile fir:
    dJ = pb + gB*B - aJ*J - hsb*cMJ*M*J - lambda*J - aJS*J*S
    
    # Equation for moose:
    dM = rM*M - beta*rM*M*M/KM + hsb*eM*cMJ*M*J - hm*M
    
    # Equation for adult spruce
    dS = rS*S - rS*S*S/KS - aSB*S*B + ps - hs*S
    return(list(c(dB,dM)))
    
  })
}

# Produce the phase plane -----

# Calculate the equilibria
wm = stode(y = c(B = 50, J = 0.1, M = 0.0105, S = 18), func = mffs, parms = PARS)$y

png("Plots/phase_plane_example.png", width = 8, height = 6, units = "in", res = 600)

# Start the plot:
par(mfrow=c(2,3), mar = c(4,4,2,1))

# BxxS ----
PARS_submod = PARS

PARS_submod = c(PARS_submod, J = unname(wm['J']), M = unname(wm['M']))

nullclines(submod_BxxS, xlim = c(0,50), ylim = c(0,50), parameters = PARS_submod, state.names = c("B", "S"), add = F, xlab = "B (blue)", ylab = "S (cyan)", add.legend = F)
points(wm[c("B")],wm[c("S")], col = "red")
flowField(submod_BxxS, xlim = c(0,50), ylim = c(0,50), parameters = PARS_submod, state.names = c("B", "S"))

# xJMx ----
PARS_submod = PARS

PARS_submod = c(PARS_submod, B = unname(wm['B']), S = unname(wm['S']))

nullclines(submod_xJMx, xlim = c(0,0.8), ylim = c(0, 0.03), parameters = PARS_submod, state.names = c("J", "M"), add = F, xlab = "J (blue)", ylab = "M (cyan)", add.legend = F)
points(wm[c("J")],wm[c("M")], col = "red")
flowField(submod_xJMx, xlim = c(0,0.8), ylim = c(0, 0.03), parameters = PARS_submod, state.names = c("J", "M"))

# BJxx ----
PARS_submod = PARS

PARS_submod = c(PARS_submod, S = unname(wm['S']), M = unname(wm['M']))

nullclines(submod_BJxx, xlim = c(0,50), ylim = c(0,0.8), parameters = PARS_submod, state.names = c("B", "J"), add = F, xlab = "B (blue)", ylab = "J (cyan)", add.legend = F)
points(wm[c("B")],wm[c("J")], col = "red")
flowField(submod_BJxx, xlim = c(0,50), ylim = c(0,0.8), parameters = PARS_submod, state.names = c("B", "J"), frac = 1)

# xxMS ----
PARS_submod = PARS

PARS_submod = c(PARS_submod, B = unname(wm['B']), J = unname(wm['J']))

nullclines(submod_xxMS, xlim = c(0, 0.03), ylim = c(0,50), parameters = PARS_submod, state.names = c("M", "S"), add = F,
           xlab = "M (blue)", ylab = "S (cyan)", add.legend = F)
points(wm[c("M")],wm[c("S")], col = "red")
flowField(submod_xxMS, xlim = c(0, 0.03), ylim = c(0,50), parameters = PARS_submod, state.names = c("M", "S"), frac = 100)

# xJxS ----
PARS_submod = PARS

PARS_submod = c(PARS_submod, B = unname(wm['B']), M = unname(wm['M']))

nullclines(submod_xJxS, xlim = c(0,0.8), ylim = c(0,50), parameters = PARS_submod, state.names = c("J", "S"), add = F, xlab = "J (blue)", ylab = "S (cyan)", add.legend = F)
points(wm[c("J")],wm[c("S")], col = "red")
flowField(submod_xJxS, xlim = c(0,0.8), ylim = c(0,50), parameters = PARS_submod, state.names = c("J", "S"), frac = 1)

# BxMx ----
PARS_submod = PARS

PARS_submod = c(PARS_submod, J = unname(wm['J']), S = unname(wm['S']))

nullclines(submod_BxMx, xlim = c(0,50), ylim = c(0, 0.03), parameters = PARS_submod, state.names = c("B", "M"), add = F, xlab = "B (blue)", ylab = "M (cyan)", add.legend = F)
points(wm[c("B")],wm[c("M")], col = "red")
flowField(submod_BxMx, xlim = c(0,50), ylim = c(0, 0.03), parameters = PARS_submod, state.names = c("B", "M"), frac = 100)

dev.off()
