









# Build the frame over time ----

# Load in the climate data:
MMAabtime <- read_csv("Data/annual_bios_NL_combined.csv") %>%
  rename(MMAname = MMA) %>%
  left_join(
    MMAframe) %>% select(-MMAname) %>%
  mutate(`bio_01` = `bio_01`/10,
         `bio_10` = `bio_10`/10,
         `bio_11` = `bio_11`/10,
         `bio_12` = `bio_12`) %>%
  arrange(Year) %>%
  filter(!is.na(MMA))

# Create the cumulative climate data: previous 29 years inform the current projection:

validyears = 1979:2099
results = vector(mode = "list", length = length(validyears))

#Better way, but just loop it for now:
for(mfy in 1:length(validyears)){
  results[[mfy]] = MMAabtime %>%
    filter(Year <= validyears[mfy] & Year >= validyears[mfy]-29) %>% pivot_longer(contains("bio")) %>% group_by(MMA,name) %>%
    summarize(value = mean(value), .groups = "drop") %>% pivot_wider() %>% ungroup() %>% mutate(Year = validyears[mfy])
  print(c(validyears[mfy]-29,validyears[mfy]))
}

MMAabtime = do.call("rbind",results) %>%
  arrange(Year)
rm(mfy, validyears, results)

MMAabtime %>% pivot_longer(contains("bio")) %>% ggplot(aes(x = Year, y = value, group = MMA)) + geom_line() + facet_wrap(~name, scales = "free")

MMAabtime = MMAabtime%>%
  mutate(
    aMM = exp(0.647 + 0.154*`bio_11` + 0.006*`bio_11`^2 -0.141*`bio_10` + 0.004*`bio_10`^2 + 2e-6*`bio_19`^2 - 0.001*`bio_19`)
  ) %>%
  mutate(summertemperature = `bio_10` - 14.6564,
         precipitation = `bio_12` - 1219.023) %>%
  select(MMA, Year,summertemperature, precipitation, aMM) %>%
  filter(!is.na(MMA)) %>%
  left_join(
    MMAabiotic %>%
      select(MMA, Qual, ps, hs, hb, hm), by = "MMA"
  ) %>% as.data.frame()

MMAabtime = cbind(MMAabtime, KB = NA, KS = NA)

for(i in 1:dim(MMAabtime)[1]){
  sitevec = c(Qual = MMAabtime[i, "Qual"],
              precipitation = MMAabtime[i, "precipitation"],
              summertemperature = MMAabtime[i, "summertemperature"],
              MMA = MMAabtime[i, "MMA"])
  
  MMAabtime[i, "KB"] = KB(sitevec)
  MMAabtime[i, "KS"] = KS(sitevec)
  
}

# Load the moose harvest rates:
mooseharvest = read_csv("Data/moose_data_over_time/moose_harvest.csv") %>% 
  mutate(MMA = ifelse(MMU == "39A", "46", MMU)) %>%
  mutate(MMA = as.numeric(MMA)) %>%
  select(-MMU) %>%
  mutate(hmr = Male*0.46 + Female*0.32) %>% # Based on weights in the script 15_mooseabundance_quotas.R
  left_join(MMAarea, by = "MMA") %>%
  mutate(hmr = hmr/area) %>%
  select(-area, -Male, -Female)

# Join harvesting with MMAabtime: 
MMAabtime = MMAabtime %>%
  full_join(
    mooseharvest, by = c("MMA", "Year")
  ) %>%
  mutate(hm = ifelse(is.na(hmr), hm, 0)) %>%
  mutate(hmr = replace_na(hmr, 0))

# Plot the forcing variables over time:
MMAabtime %>% pivot_longer(!Year & !MMA) %>% ggplot(aes(x = Year, y = value, group = MMA)) + geom_line() + facet_wrap(~name, scales = "free")

# The function that includes time effects
mffs_time <- function(t, y, pars){
  with(as.list(c(pars,y)),{
    # Equation for adult fir:
    dB = lambda*J + rB*B - rB*B*B/KB_t(t) - aBS*B*S - hb*B
    
    # Equation for juvenile fir:
    dJ = pb + gB*B - aJ*J - hsb*p*cMJ*M*J - lambda*J - aJS*J*S
    
    # Equation for moose:
    dM = (1-p)*rM*M + hsb*p*eM*cMJ*M*J - p*beta*aMM_t(t)*M*M - hm_t(t)*M
    # NOTE: In the MS, I use the notation rho*hm*M for the hunting rate, since it makes sense to have hunting only happen in the winter. The data we have estimates annual hunting rate, so I just leave out rho in this version.
    
    # Equation for adult spruce
    dS = rS*S - rS*S*S/KS_t(t) - aSB*S*B + ps - hs*S
    
    return(list(c(dB, dJ, dM, dS)))
    
  })
}

# Loop over the MMAs:

results <- vector(mode = "list", length = dim(MMAabiotic2)[1])

intpfcns <- vector(mode = "list", length = dim(MMAabiotic2)[1])


# Create the interpolated functions for hm,aMM, KB, and KS.
for(i in 1:length(results)){
  
  # These functions help with spin-up, because they simulate over 1950 to 1965 three times to stabilize the dynamics before the simualtions are run. I chose 1965 because the moose data start in 1968 and this made sense to spin-up before entering the dynamics where we have data.
  
  MMAtimetemp = subset(MMAabtime, MMA == i) %>% 
    arrange(Year)
  
  MMAtimetemp =  MMAtimetemp %>%
    filter(Year < 1995) %>%
    bind_rows(
      MMAtimetemp %>%
        filter(Year < 1995)
    ) %>%
    bind_rows(
      MMAtimetemp %>%
        filter(Year < 1995)
    ) %>%
    bind_rows(
      MMAtimetemp %>%
        filter(Year < 1995)
    ) %>%
    bind_rows(
      MMAtimetemp %>%
        filter(Year < 1995)
    ) %>%
    bind_rows(
      MMAtimetemp
    ) %>%
    mutate(modtime = 1:201)
  
  aMM_t = approxfun(MMAtimetemp$modtime,MMAtimetemp$aMM)
  KB_t = approxfun(MMAtimetemp$modtime,MMAtimetemp$KB)
  KS_t = approxfun(MMAtimetemp$modtime,MMAtimetemp$KS)
  hm_t = approxfun(MMAtimetemp$modtime,MMAtimetemp$hm)
  
  intpfcns[[i]] = list(aMM_t, KB_t, KS_t, hm_t)
}
rm(KB_t, KS_t, hm_t)

# Create time to year association
MMAtimetemp = MMAtimetemp %>%
  select(Year, modtime) %>%
  # ID the spin-up
  mutate(Year = ifelse(modtime < 65, -Year, Year)) %>%
  rename(time = modtime)

# Run the models
for(i in 1:length(results)){
  PARSi = PARS
  PARSi["beta"] = 350
  for(iname in colnames(MMAabiotic2)[-1]){
    PARSi[iname] = MMAabiotic2[i,iname]
  }
  
  eqmi = stode(y = c(B = 100, J = 0.1, M = 0.1, S = 100), func = mffs, parms = PARSi)
  
  if(max(Re(eigen(jacobian.full(y = eqmi$y, func = mffs, parms = PARSi))$values)) > 0) print(paste0(i, " is not steady."))
  
  if(!attributes(eqmi)$steady) print(paste0(i, " is not steady."))
  
  
  
  simres = ode(y = eqmi$y, times= 1:200, func = mffs_time, 
               # Load the interpolated functions for aMM, KB, KS, and hm.
               parms = c(PARSi, 
                         aMM_t = intpfcns[[i]][[1]], 
                         KB_t = intpfcns[[i]][[2]],
                         KS_t = intpfcns[[i]][[3]], 
                         hm_t = intpfcns[[i]][[4]]))
  
  results[[i]] = cbind(as.data.frame(simres), MMA = i)
  print(paste("Done MMA", i))
}

results = do.call("rbind",results) %>%
  tibble() %>%
  # Add in the years
  left_join(
    MMAtimetemp
  ) %>%
  # Cut out the spin-up
  filter(Year > 0) %>%
  select(-time)


results %>%
  pivot_longer(!Year & !MMA) %>% 
  ggplot(aes(x = Year, y = value, group = MMA)) + geom_line() + facet_wrap(.~name, scales = "free")

# Fit moose growth parameters ----

# Get the moose population data

moosedensity2 = read_rds("Data/moosedensity_raw.rds") %>%
  rename(MMAname = MMA) %>%
  left_join(
    MMAframe, by = "MMAname"
  ) %>% select(-MMAname)

moosedensity2 %>%
  left_join(
    results, by = c("MMA", "Year")
  ) %>%
  ggplot(aes(x = M, y = Moose)) + geom_point() + stat_regline_equation(aes(label =  paste(..eq.label.., ..adj.rr.label.., sep = "~~~~")))

results %>%
  # Cut out the spin-up
  filter(Year > 0 & Year < 2025) %>%
  ggplot(aes(x = Year,y = M, group = MMA)) + geom_line() + theme_classic() + 
  geom_point(data = moosedensity2, aes(y = Moose)) + 
  facet_wrap(.~MMA)

# .. Fit the moose population parameters ----

fitmoosepars <- function(ptf, PARif = PARS, md = moosedensity2, intpfcns2 = intpfcns, timetoyear = MMAtimetemp){
  
  PARif["beta"] = exp(ptf[1])
  PARif["rM"] = exp(ptf[2])
  
  results <- vector(mode = "list", length = dim(MMAabiotic2)[1])
  
  # Run the models
  for(i in 1:length(results)){
    PARSi = PARif
    for(iname in colnames(MMAabiotic2)[-1]){
      PARSi[iname] = MMAabiotic2[i,iname]
    }
    
    eqmi = stode(y = c(B = 100, J = 0.1, M = 0.1, S = 100), func = mffs, parms = PARSi)
    
    if(max(Re(eigen(jacobian.full(y = eqmi$y, func = mffs, parms = PARSi))$values)) > 0) print(paste0(i, " is not steady."))
    
    if(!attributes(eqmi)$steady) print(paste0(i, " is not steady."))
    
    simres = ode(y = eqmi$y, times= 1:200, func = mffs_time, 
                 # Load the interpolated functions for aMM, KB, KS, and hm.
                 parms = c(PARSi, 
                           aMM_t = intpfcns[[i]][[1]], 
                           KB_t = intpfcns[[i]][[2]],
                           KS_t = intpfcns[[i]][[3]], 
                           hm_t = intpfcns[[i]][[4]]))
    
    results[[i]] = cbind(as.data.frame(simres), MMA = i)
  }
  
  md %>%
    left_join(
      do.call("rbind",results) %>%
        tibble() %>%
        left_join(
          timetoyear, by = c("time")
        ), by = c("MMA", "Year")
    ) %>%
    mutate(diff = (100*Moose - 100*M)^2) %>%
    summarize(sum(diff)) %>% pull()
}
debugonce(fitmoosepars)
fitmoosepars(log(c(1072.136790,1.121182)))

fitmoosepars(log(c(350,1.121182)))

fitmoosepars(c(5.7213294,0.4111488))

system.time(fitmoosepars(log(c(1072.136790,1.121182))))

optmoose1 <- optim(log(c(1072.136790,1.121182)), fitmoosepars)

# $par
# [1] 5.7213294 0.4111488
# 
# $value
# [1] 39.71773
# Convergence 10

exp(c(5.7213294, 0.4111488))

optmoose2 <- optim(log(c(100,2)), fitmoosepars)

# $par
# [1] 5.715867 0.406834
# 
# $value
# [1] 39.71777
# 
# $counts
# function gradient 
# 313       NA 
# 
# $convergence
# [1] 0
# 
# $message
# NULL

exp(c(5.715867, 0.406834))



# Loop over the MMAs ----
PARSnew = PARS

PARSnew["beta"] = 303.647351
PARSnew["rM"] = 1.502055


results <- vector(mode = "list", length = dim(MMAabiotic2)[1])

# Run the models
for(i in 1:length(results)){
  PARSi = PARSnew
  for(iname in colnames(MMAabiotic2)[-1]){
    PARSi[iname] = MMAabiotic2[i,iname]
  }
  
  eqmi = stode(y = c(B = 100, J = 0.1, M = 0.1, S = 100), func = mffs, parms = PARSi)
  
  if(max(Re(eigen(jacobian.full(y = eqmi$y, func = mffs, parms = PARSi))$values)) > 0) print(paste0(i, " is not steady."))
  
  if(!attributes(eqmi)$steady) print(paste0(i, " is not steady."))
  
  # Load the interpolated functions for aMM, KB, and KS.
  
  aMM_t = intpfcns[[i]][[1]]
  KB_t = intpfcns[[i]][[2]]
  KS_t = intpfcns[[i]][[3]]
  
  simres = ode(y = eqmi$y, times= 1:229, func = mffs_time, parms = PARSi)
  
  results[[i]] = cbind(as.data.frame(simres), MMA = i)
}

# Figure for Isabelle
if(F){
  ddd = do.call("rbind",results) %>%
    tibble() %>%
    mutate(Year = ifelse(time > 80, time + 1869, NA)) %>%
    filter(MMA == 1)
  
  ddd %>%
    ggplot(aes(x = time, y = M)) + geom_line() + theme_classic() + 
    geom_text(aes(x = time, y = 0.02, label = Year),
              data = ddd %>%
                filter(Year %in% seq(1950, 2100, by = 40))) +
    geom_vline(xintercept = 80, size = 1, color = "red") +
    geom_text(aes(x= time, y = 0.022, label = label),
              data = tibble(time = 30, label = "Spin-up\n (looping 1950's)")) +
    geom_text(aes(x= time, y = 0.024, label = label),
              data = tibble(time = 125, label = "Simulation period")) +
    ylab("Moose (Mg/ha)")
}

results = do.call("rbind",results) %>%
  tibble() %>%
  filter(time > 80) %>%
  mutate(Year = time + 1869) %>%
  select(-time)

png("Plots/balsam_fir_trajectories.png", width = 6, height = 6, units = "in", res = 600)
results %>%
  filter(MMA %in% c(1,10,30,40)) %>%
  mutate(MMA = paste("MMA", MMA)) %>%
  ggplot(aes(x = Year, y = B)) + 
  geom_line() + facet_wrap(~MMA) + ylab("Balsam fir (Mg/ha)")
dev.off()


# Plot the results ----

png(paste0("Plots/Rempel/simulation_fit.png"), width = 8, height = 5, units = "in", res = 600)
moosedensity2 %>%
  left_join(
    results, by = c("MMA", "Year")
  ) %>%
  ggplot(aes(x = M, y = Moose)) + geom_point() + stat_regline_equation(aes(label =  paste(..eq.label.., ..adj.rr.label.., sep = "~~~~"))) + theme(legend.position = "none") + ylab("Data") + xlab("Model") + ggtitle("Moose model vs data comparison")
dev.off()

png(paste0("Plots/Rempel/simulation_fit_time.png"), width = 10, height = 10, units = "in", res = 600)
results %>%
  pivot_longer(!Year & !MMA) %>%
  filter(name == "M") %>%
  ggplot(aes(x = Year, y = value)) + geom_line() + facet_wrap(.~MMA) + theme_classic() + ylab("Moose (t/ha)") +
  geom_point(aes(x = Year, y = Moose), data = moosedensity2)
dev.off()

png(paste0("Plots/Rempel/simulation_example_024.png"), width = 10, height = 10, units = "in", res = 600)

cowplot::plot_grid(
  results %>%
    pivot_longer(!Year & !MMA) %>%
    filter(name == "M") %>%
    filter(MMA == 24) %>%
    ggplot(aes(x = Year, y = value)) + geom_line() + facet_wrap(.~MMA) + theme_classic() + ylab("Moose (t/ha)") +
    geom_point(aes(x = Year, y = Moose), data = moosedensity2 %>% filter(MMA == 24)),
  results %>%
    pivot_longer(!Year & !MMA) %>%
    filter(name == "B") %>%
    filter(MMA == 24) %>%
    ggplot(aes(x = Year, y = value)) + geom_line() + ylab("Balsam Fir (t/ha)") + theme_classic(),
  read_csv("Data/annual_bios_NL_combined.csv") %>%
    filter(MMA == "024") %>%
    ggplot(aes(x = Year, y = `bio_01`/10)) + geom_line() + theme_classic() + ylab("Annual mean temperature"),
  read_csv("Data/annual_bios_NL_combined.csv") %>%
    filter(MMA == "024") %>%
    ggplot(aes(x = Year, y = `bio_12`))  + geom_line() + theme_classic() + ylab("Total annual precip.")
)
dev.off()

png(paste0("Plots/Rempel/simulation_1950_2098.png"), width = 8, height = 5, units = "in", res = 600)
results %>%
  pivot_longer(!Year & !MMA) %>%
  ggplot(aes(x = Year, y = value, color = as.factor(MMA))) + geom_line() + facet_wrap(.~name, scales = "free") + theme_classic()+ theme(legend.position = "none")
dev.off()

results %>%
  pivot_longer(!Year & !MMA) %>%
  write_csv("Data/time_series_prediction.csv")