# Simulate the model over time:
# A: Robert W. Buchkowski
# D: Jan 19/2023

# Load in the packages:
if(!require("pacman")) install.packages("pacman")
p_load(raster, FME, phaseR, tidyverse, ggpubr, sf, googledrive,nleqslv)

# Load in the model parameters ----

# Parameters
PARS <- c(aBS = 1e-3, # Fit in last model
          aJS = 1e-3, # Same as aBS
          aSB = 1e-3, # Fit in last model
          rB = 0.2, # Lavigne 1985
          gB = 0.011, # Rossi and Morin 2011
          hsb = 1, # the bias moose have towards commercial forest patches within the MMA: high numbers means higher moose browsing and resource benefits from juvenile fir
          eM = 0.4, # Renecker and Hudson 1986
          cMJ = 2.33*0.5, # Crete 1989 and only occurs in winter
          aJ = 0.2, # Wang and Kemball 2005
          rS = 0.1, # 50% of rB based on Lemay et al. 2018
          rM = 0.1816976, # Fryxell et al. 1988
          KM = 0.7983134, # Calculated from Rempel 2011
          beta = 75.9017574, # fit below, this is used to correct KM to match NFLD
          hm = 1.494064e-01, # Hunting rate (per year)
          hs = 7.124689e-03,
          hb = 4.692526e-03,
          pb = 0,
          ps = 1.193254e-06,
          lambda = 0.464, # Nonoon et al. 2020
          lambda_slope = -1.53, # Nonoon et al. 2020, assume linear relationship
          KS = 8.068953e+00,
          KB = 1.688549e+01)

# Non-browsable balsam fir
KB <- function(site){
  estK = exp(2.841701250   + 
               0.003118377*site["precipitation"] -
               0.242044069*site["summertemperature"]+
               0.458836267*(site["Qual"]== 2) + 
               1.228691625*(site["Qual"]== 3) + 
               2.088775004*(site["Qual"]== 4)-
               0.405253062*(site["MMA"]==2)+
               0.355428768*(site["MMA"]==3)+
               0.129214844*(site["MMA"]==4)-
               0.299552199*(site["MMA"]==5)-
               0.560501432*(site["MMA"]==6)-
               0.713942917*(site["MMA"]==7)-
               1.008606069*(site["MMA"]==8)-
               1.436008346*(site["MMA"]==9)-
               1.676413452*(site["MMA"]==10)-
               1.239127530*(site["MMA"]==11)-
               0.829188464*(site["MMA"]==12)-
               1.926652939*(site["MMA"]==13)-
               0.754021526*(site["MMA"]==14)-
               1.426591267*(site["MMA"]==15)-
               1.488102742*(site["MMA"]==16)-
               1.181664060*(site["MMA"]==17)-
               1.140222984*(site["MMA"]==18)-
               1.614786343*(site["MMA"]==19)-
               2.352607667*(site["MMA"]==20)-
               1.920056804*(site["MMA"]==21)-
               1.876718012*(site["MMA"]==22)-
               2.332777516*(site["MMA"]==23)-
               2.312415976*(site["MMA"]==24)-
               2.080964233*(site["MMA"]==25)-
               3.182677523*(site["MMA"]==26)-
               2.694996805*(site["MMA"]==27)-
               1.832385553*(site["MMA"]==28)-
               1.764568810*(site["MMA"]==29)-
               0.904897609*(site["MMA"]==30)-
               0.611285585*(site["MMA"]==31)-
               0.809262315*(site["MMA"]==32)-
               0.909772969*(site["MMA"]==33)-
               0.891546740*(site["MMA"]==34)-
               0.911647246*(site["MMA"]==35)-
               1.041803691*(site["MMA"]==36)-
               1.170314685*(site["MMA"]==37)-
               1.104167475*(site["MMA"]==38)+
               0.117449315*(site["MMA"]==39)+
               0.193874540*(site["MMA"]==40)-
               1.348057087*(site["MMA"]==41)-
               2.078589614*(site["MMA"]==42)-
               0.853558869*(site["MMA"]==43)-
               0.554977736*(site["MMA"]==44)+
               0.154724014*(site["MMA"]==45)+
               0.645984203*(site["MMA"]==46)-
               0.756895927*(site["MMA"]==47)
  )
  
  unname(estK)
}

# Spruce
KS <- function(site){
  estK = exp(1.222346455 - 
               0.001813901*site["precipitation"] +
               0.244429935*site["summertemperature"] -
               0.871367479*(site["Qual"]== 2) - 
               1.387239611*(site["Qual"]== 3) - 
               1.153662224*(site["Qual"]== 4) +
               0.194458882*(site["MMA"]==2)+
               2.081076722*(site["MMA"]==3)+
               1.620965342*(site["MMA"]==4)+
               0.725292400*(site["MMA"]==5)+
               0.827422524*(site["MMA"]==6)+
               0.845362182*(site["MMA"]==7)+
               0.816090465*(site["MMA"]==8)+
               0.793501297*(site["MMA"]==9)+
               0.788531090*(site["MMA"]==10)+
               1.450851267*(site["MMA"]==11)+
               2.033932902*(site["MMA"]==12)+
               2.056431924*(site["MMA"]==13)+
               1.601457045*(site["MMA"]==14)+
               1.597160291*(site["MMA"]==15)+
               1.948481377*(site["MMA"]==16)+
               2.464766692*(site["MMA"]==17)+
               2.191841469*(site["MMA"]==18)+
               1.530759972*(site["MMA"]==19)+
               3.723391833*(site["MMA"]==20)+
               2.204426935*(site["MMA"]==21)+
               1.661226038*(site["MMA"]==22)+
               2.062910023*(site["MMA"]==23)+
               2.338305244*(site["MMA"]==24)+
               1.882400051*(site["MMA"]==25)+
               2.049079657*(site["MMA"]==26)+
               2.693936133*(site["MMA"]==27)+
               2.129416615*(site["MMA"]==28)+
               2.304136111*(site["MMA"]==29)+
               0.088531427*(site["MMA"]==30)+
               0.672894451*(site["MMA"]==31)+
               0.167968013*(site["MMA"]==32)+
               0.379789681*(site["MMA"]==33)+
               0.720403912*(site["MMA"]==34)+
               0.566346286*(site["MMA"]==35)+
               0.663662650*(site["MMA"]==36)+
               1.163171261*(site["MMA"]==37)-
               0.006194267*(site["MMA"]==38)+
               1.856394600*(site["MMA"]==39)+
               0.933942243*(site["MMA"]==40)+
               1.725395646*(site["MMA"]==41)+
               2.520176458*(site["MMA"]==42)+
               0.205567771*(site["MMA"]==43)+
               0.367126102*(site["MMA"]==44)+
               0.580090667*(site["MMA"]==45)+
               1.190611613*(site["MMA"]==46)+
               2.370165384*(site["MMA"]==47)
  )
  
  unname(estK)
}

# Create the MMA identity data frame so we can graph the results
MMAframe = tibble(MMA = c(1:45, 46, 47),
                  MMAname = c(paste0("00", 1:9), 
                              paste0("0", 10:45),
                              "039A", "047"))
  
# Get the average moose harvesting rate:
mooseharvest = read_csv("Data/moose_populationANDharvest_rate.csv") %>%
filter(!is.na(harvest_mpe)) %>%
group_by(MMA) %>%
summarize(hm = mean(harvest_mpe))

mooseharvest = mooseharvest %>%
filter(MMA == "39/39A") %>%
mutate(MMA = "039") %>%
bind_rows(
  mooseharvest %>%
    filter(MMA == "39/39A") %>%
    mutate(MMA = "039A")
) %>% 
bind_rows(
  mooseharvest %>%
    filter(MMA != "39/39A")
) %>%
mutate(MMA = str_pad(MMA, width=3, side='left', pad='0'))

# Put together the MMA properties into a data frame:
MMAabiotic <- MMAframe %>% 
inner_join(
  read_rds("Data/MMA_abiotic_variables.rds") %>%
    mutate(
      KM = PARS["rB"]/(exp(0.647 + 0.154*bio11 + 0.006*bio11^2 -0.141*bio10 + 0.004*bio10^2 + 2e-6*bio19^2 - 0.001*bio19))
    ) %>%
    left_join(
      tibble(Qual = 1:4,
             Qualmode = factor(c("P", "M", "G", "H")))
    ) %>%
    select(MMA, Qual, summertemperature, precipitation, KM) %>%
    
    # Read in the hunting and harvesting and planting data by MMA
    full_join(
      read_rds("Data/harvest_by_MMA.rds") %>%
        rename(ps = ps_med, 
               hs = propAAC_S,
               hb = propAAC_B) %>%
        select(MMA, ps, hs, hb)
    ) %>%
    full_join(
      mooseharvest
    ) %>% 
    rename(MMAname = MMA)) %>% 
select(-MMAname) %>% as.data.frame()

rm(mooseharvest) # Clear moose harvest


MMAabiotic = cbind(MMAabiotic, KB = NA, KS = NA)

for(i in 1:dim(MMAabiotic)[1]){
sitevec = c(Qual = MMAabiotic[i, "Qual"],
            precipitation = MMAabiotic[i, "precipitation"],
            summertemperature = MMAabiotic[i, "summertemperature"],
            MMA = MMAabiotic[i, "MMA"])

MMAabiotic[i, "KB"] = KB(sitevec)
MMAabiotic[i, "KS"] = KS(sitevec)

}

MMAabiotic2 = MMAabiotic %>%
select(MMA, KM, KB, KS, ps, hs, hb, hm)

summary(MMAabiotic2)

# Load in the functions at equilibrium ----

#Model with moose:
mffs <- function(t, y, pars){
  with(as.list(c(pars,y)),{
    # Equation for adult fir:
    dB = (lambda + lambda_slope*M)*J + rB*B - rB*B*B/KB - aBS*B*S - hb*B
    
    # Equation for juvenile fir:
    dJ = pb + gB*B - aJ*J - hsb*cMJ*M*J - (lambda + lambda_slope*M)*J - aJS*J*S
    
    # Equation for moose:
    dM = rM*M - beta*rM*M*M/KM + hsb*eM*cMJ*M*J - hm*M
    
    # Equation for adult spruce
    dS = rS*S - rS*S*S/KS - aSB*S*B + ps - hs*S
    
    return(list(c(dB, dJ, dM, dS)))
    
  })
}

# Model without moose:
ffs <- function(t, y, pars){
  with(as.list(c(pars,y)),{
    # Equation for adult fir:
    dB = lambda*J + rB*B - rB*B*B/KB - aBS*B*S - hb*B
    
    # Equation for juvenile fir:
    dJ = pb + gB*B  - aJ*J - lambda*J - aJS*J*S
    
    # Equation for adult spruce
    dS = rS*S - rS*S*S/KS - aSB*S*B + ps - hs*S
    
    return(list(c(dB, dJ, dS)))
    
  })
}

#Model without spruce:
mff <- function(t, y, pars){
  with(as.list(c(pars,y)),{
    # Equation for adult fir:
    dB = (lambda + lambda_slope*M)*J + rB*B - rB*B*B/KB - hb*B
    
    # Equation for juvenile fir:
    dJ = pb + gB*B  - aJ*J - hsb*cMJ*M*J - (lambda + lambda_slope*M)*J
    
    # Equation for moose:
    dM = rM*M - beta*rM*M*M/KM + hsb*eM*cMJ*M*J - hm*M
    
    return(list(c(dB, dJ, dM)))
    
  })
}

# Build a function to analyze the model by MMA:

RUNbyMMA <- function(model, # The model to run 
                     constpars, # The constant parameters 
                     varpars,# The variable parameters
                     startvec = c(B = 100, J = 0.1, M = 0.1, S = 100),
                     UID = "MMA",
                     reteigenvalue = F){ 
  
  results = vector(mode = "list", length = dim(varpars)[1])
  
  for(i in 1:length(results)){
    constparsi = constpars
    for(iname in colnames(varpars)){
      constparsi[iname] = varpars[i,iname]
    }
    
    eqmi = stode(y = startvec, func = model, parms = constparsi)
    
    if(max(Re(eigen(jacobian.full(y = eqmi$y, func = model, parms = constparsi))$values)) > 0) print(paste0(i, " is not steady."))
    
    if(!attributes(eqmi)$steady) print(paste0(i, " is not steady."))
    
    if(reteigenvalue){
      output = c(tempor = i, Re(eigen(jacobian.full(y = eqmi$y, func = model, parms = constparsi))$values))
      names(output)[1] = UID
    }else{
      output = c(tempor = i, eqmi$y)
      names(output)[1] = UID
    }
    results[[i]] = output
  }
  return(as.data.frame(do.call("rbind",results)))
}

# Fit the moose parameter ----

# Get the MMA area:
MMA <- read_sf("C:/Users/rober/Documents/Moose_Model_Original_Data/MMA/MMA.shp") %>%
  mutate(MMA = ifelse(MMA == "000", "006", MMA)) %>%
  group_by(MMA) %>% 
  summarize() %>%
  ungroup()

MMAarea = tibble(MMAname = MMA$MMA,
                 area = as.numeric(st_area(MMA))/10000) %>% # hectare
  left_join(MMAframe) %>%
  filter(!is.na(MMA)) %>%
  select(-MMAname)

MMA = MMA %>%
  rename(MMAname = MMA) %>%
  left_join(MMAframe) %>%
  filter(!is.na(MMA)) %>%
  select(MMA)


# Read in the moose population data:
moosepop = read_csv("Data/moose_populationANDharvest_rate.csv") %>%
  filter(!is.na(harvest_mpe))

moosepop = moosepop %>%
  filter(MMA == "39/39A") %>%
  mutate(MMA = "039") %>%
  bind_rows(
    moosepop %>%
      filter(MMA == "39/39A") %>%
      mutate(MMA = "039A")
  ) %>% 
  bind_rows(
    moosepop %>%
      filter(MMA != "39/39A")
  ) %>%
  mutate(MMA = str_pad(MMA, width=3, side='left', pad='0'))

moosepop = MMAframe %>%
  left_join(
    moosepop %>% rename(MMAname = MMA)
  ) %>%
  left_join(
    MMAarea
  ) %>%
  filter(!is.na(ModelPopEst) &
           Year > 1980) %>% # Keep only moose population data from the time when the climate data matches
  group_by(MMA) %>%
  mutate(ModelPopEst = ModelPopEst*0.35/area) %>% # Convert to biomass per hectare (Mg/ha)
  summarize(mooseavg = mean(ModelPopEst), moosesd = sd(ModelPopEst), N = n())

# Compare the model results with the moose population data:

summary(moosepop)

# Only need to run the fit the first time. The best fit parameter is already listed above
if(F){
  
  empdata = read_rds("Data/FRI_trees_by_MMA.rds") %>%
    left_join(
      tibble(ID = c("spruce", "fir", "juvfir"),
             name = c("S", "B", "J"))
    ) %>% rename(MMAname = MMA) %>%
    full_join(MMAframe) %>%
    select(MMA, name, mean, SD) %>%
    bind_rows(
      moosepop %>%
        rename(mean = mooseavg, SD = moosesd) %>%
        mutate(name = "M")
    ) %>%
    mutate(N = replace_na(N, 1)) %>% 
    left_join(
      tibble(name = c("B", "J", "M", "S"),
             scalepar = c(20.6, 0.0182, 0.00294, 17.1)) # Standard deviation
    )
  
  
  empdata  %>%
    full_join(
      # Run the model
      tibble(RUNbyMMA(mffs, PARS, MMAabiotic2)) %>%
        pivot_longer(-MMA), by = c("MMA", "name")
    ) %>%
    mutate(SE = (value/scalepar-mean/scalepar)^2, scaletest = mean/scalepar) %>%
    group_by(name) %>%
    summarize(mean(SE), mean(scaletest))
  
  fitbeta <- function(logbeta, PARSin = PARS, empdatain = empdata){
    PARSin["beta"] = exp(logbeta[1])
    PARSin["rM"] = exp(logbeta[2])
    
    empdatain  %>%
      full_join(
        # Run the model
        tibble(RUNbyMMA(mffs, PARSin, MMAabiotic2)) %>%
          pivot_longer(-MMA), by = c("MMA", "name")
      ) %>%
      mutate(SE = (value/scalepar-mean/scalepar)^2) %>% # The multiplier just puts the difference in better units
      summarise(SSE = sum(SE)) %>% pull()
  }
  
  startvec = log(c(500, 1.37))
  fitbeta(startvec)
  
  opout = optim(startvec,fitbeta)
  
  exp(opout$par)
}

# Run the model at equilibrium ----

modeldata = read_rds("Data/FRI_trees_by_MMA.rds") %>%
  left_join(
    tibble(ID = c("spruce", "fir", "juvfir"),
           name = c("S", "B", "J"))
  ) %>% rename(MMAname = MMA) %>%
  full_join(MMAframe) %>%
  select(MMA, name, mean, SD) %>%
  bind_rows(
    moosepop %>%
      rename(mean = mooseavg, SD = moosesd) %>%
      mutate(name = "M")
  ) %>%
  mutate(N = replace_na(N, 1)) %>%
  full_join(
    # Run the model
    tibble(RUNbyMMA(mffs, PARS, MMAabiotic2)) %>%
      pivot_longer(-MMA)
  )

pred_vs_data = modeldata %>%
  ggplot(aes(x = value, y = mean)) + geom_pointrange(aes(ymin = mean-SD, ymax = mean + SD), color = "grey") + theme_classic() + facet_wrap(.~name, scales = "free", nrow = 1) + geom_abline(intercept = 0, slope = 1, linetype = 2, color = "grey20") + ylab("Empirical measurements") + xlab("Model predictions") +
  stat_regline_equation(label.x.npc = 0.1, label.y.npc = 0.95,aes(label = ..eq.label..)) +
  stat_regline_equation(label.x.npc = 0.1, label.y.npc = 0.85, aes(label = ..rr.label..))

pred_vs_data

modeldatamap = MMA %>% 
  left_join(modeldata)

png(paste0("Plots/moose_alter_transition/maps_model_pred_MMA.png"), width = 8, height = 8, units = "in", res = 600)
cowplot::plot_grid(
  cowplot::plot_grid(
    modeldatamap %>%
      filter(name == "B") %>%
      ggplot() + geom_sf(aes(fill = value)) + theme_classic() +
      scale_fill_viridis_c(name = "Balsam Fir\n(Mg/ha)"),
    
    modeldatamap %>%
      filter(name == "J") %>%
      ggplot() + geom_sf(aes(fill = value)) + theme_classic() +
      scale_fill_viridis_c(name = "Juv. Balsam\nFir (Mg/ha)"),
    
    modeldatamap %>%
      filter(name == "M") %>%
      ggplot() + geom_sf(aes(fill = value)) + theme_classic() +
      scale_fill_viridis_c(name = "Moose\n(Mg/ha)"),
    
    modeldatamap %>%
      filter(name == "S") %>%
      ggplot() + geom_sf(aes(fill = value)) + theme_classic() +
      scale_fill_viridis_c(name = "Spruce\n(Mg/ha)")
  ),
  pred_vs_data, ncol = 1, nrow = 2, rel_heights = c(3,1)
)
dev.off()

# Plot the model moose effects by MMA ----

mooseeffectmap = MMA %>% 
  left_join(
    tibble(RUNbyMMA(mffs, PARS, MMAabiotic2)) %>%
      mutate(Moose = "Yes") %>%
      bind_rows(
        tibble(RUNbyMMA(ffs, PARS, MMAabiotic2, c(B = 100, J = 0.1, S = 100))) %>%
          mutate(Moose = "No") 
      ) %>%
      select(-M) %>%
      pivot_longer(B:S) %>%
      pivot_wider(names_from = Moose) %>%
      mutate(EOM = 100*(Yes-No)/No) %>%
      left_join(
        tibble(name = c("B", "J", "S", "M"),
               name2 = c("Balsam fir", "Juv. Balsam fir", "Spruce", "Moose"))
      )
  )

modeldata %>%
  full_join(
    mooseeffectmap %>%
      tibble() %>%
      select(MMA, name, EOM)
  ) %>%
  write_rds("Data/Data_for_model_version_compare/modeldata_lambdamod.rds")

png(paste0("Plots/moose_alter_transition/moose_effect_MMA.png"), width = 8, height = 8, units = "in", res = 600)
cowplot::plot_grid(
  mooseeffectmap %>%
    filter(name == "B") %>%
    ggplot() + geom_sf(aes(fill = EOM)) + theme_classic() +
    scale_fill_viridis_c(name = "Change in\nBalsam Fir (%)"),
  
  mooseeffectmap %>%
    filter(name == "J") %>%
    ggplot() + geom_sf(aes(fill = EOM)) + theme_classic() +
    scale_fill_viridis_c(name = "Change in Juv.\nBalsam Fir (%)"),
  
  mooseeffectmap %>%
    filter(name == "S") %>%
    ggplot() + geom_sf(aes(fill = EOM)) + theme_classic() +
    scale_fill_viridis_c(name = "Change in\nSpruce (%)")
)
dev.off()

mooseeffectmap %>%
  tibble() %>% select(name2, EOM) %>%
  group_by(name2) %>%
  summarize(min(EOM), median(EOM), max(EOM))

rm(mooseeffectmap)

# Bifurcation MMA: Management parameters ----

tempgridhm <- expand.grid(hb = mean(MMAabiotic2$hm),
                          hm = seq(0,max(MMAabiotic2$hm), length = 10),
                          hs = mean(MMAabiotic2$hs),
                          ps = mean(MMAabiotic2$ps),
                          MMA = 1:47)

tempgridhb <- expand.grid(hb = seq(0,max(MMAabiotic2$hb), length = 10),
                          hm = mean(MMAabiotic2$hm),
                          hs = mean(MMAabiotic2$hs),
                          ps = mean(MMAabiotic2$ps),
                          MMA = 1:47)

tempgridhs <- expand.grid(hb = mean(MMAabiotic2$hb),
                          hm = mean(MMAabiotic2$hm),
                          hs = seq(0,max(MMAabiotic2$hs), length = 10),
                          ps = mean(MMAabiotic2$ps),
                          MMA = 1:47)

tempgridps <- expand.grid(hb = mean(MMAabiotic2$hb),
                          hm = mean(MMAabiotic2$hm),
                          ps = seq(0,max(MMAabiotic2$ps), length = 10),
                          hs = mean(MMAabiotic2$hs),
                          MMA = 1:47)

plot_range <- function(tempgridcur, PARM){
  tempgridcur = tempgridcur %>%
    left_join(
      MMAabiotic2 %>% 
        select(-hm, -hb, -hs, -ps), # Remove the parameters for each MMA
      by = "MMA"
    )
  
  tempgrad = tibble(RUNbyMMA(mffs, PARS, tempgridcur)) %>%
    select(-MMA) %>%
    bind_cols(tempgridcur) %>%
    mutate(Moose = "Yes") %>%
    bind_rows(
      tibble(RUNbyMMA(ffs, PARS, tempgridcur, c(B = 100, J = 0.1, S = 100))) %>%
        select(-MMA) %>%
        bind_cols(tempgridcur) %>%
        mutate(Moose = "No") 
    ) %>%
    select(-M) %>%
    pivot_longer(B:S) %>%
    pivot_wider(names_from = Moose) %>%
    mutate(EOM = 100*(Yes-No)/No) %>%
    left_join(
      tibble(name = c("B", "J", "S", "M"),
             name2 = c("Balsam fir", "Juv. Balsam fir", "Spruce", "Moose"))
    )
  
  
  colnames(tempgrad)[colnames(tempgrad) == PARM] = "CURPARM"
  
  tempgrad2 = tempgrad %>%
    group_by(CURPARM, name2) %>%
    rename(value = EOM) %>%
    summarize(mean = mean(value), sd = sd(value), min = min(value), max = max(value), lq = quantile(value, 0.25), uq = quantile(value, 0.75), median = median(value))
  
  return(tempgrad2)
}

hmrange <- plot_range(tempgridhm,"hm")
hbrange <- plot_range(tempgridhb,"hb")
hsrange <- plot_range(tempgridhs,"hs")
psrange <- plot_range(tempgridps,"ps")

png(paste0("Plots/moose_alter_transition/mooseeffect_mgmtparams1.png"), width = 8, height = 6, units = "in", res = 600)
hmrange %>%
  mutate(CURPARM = CURPARM, Parameter = "Moose harvest rate") %>%
  bind_rows(
    hbrange %>%
      mutate(CURPARM = CURPARM, Parameter = "Balsam fir harvest rate")
  ) %>%
  bind_rows(
    hsrange %>%
      mutate(CURPARM = CURPARM, Parameter = "Spruce harvest rate")
  ) %>%
  bind_rows(
    psrange %>%
      mutate(CURPARM = CURPARM, Parameter = "Spruce planing rate")
  ) %>%
  ggplot(aes(x = CURPARM, y = median, color = name2, fill = name2)) + geom_ribbon(aes(ymin = lq, ymax = uq), alpha = 0.5) + facet_wrap(Parameter~name2, scales = "free", nrow = 4, ncol = 3) +
  geom_line(size = 2) + theme_classic() + ylab("Moose effect (%)") + xlab("Parameter value") + theme(legend.position = "none")
dev.off()

# Bifurcation: Moose habitat selection bias ----

tempgrid <- expand.grid(hsb = c(0.1, 0.5, 1, 5, 10), 
                        MMA = 1:47) %>%
  left_join(
    MMAabiotic2, by = "MMA"
  )

tempresults <- cbind(RUNbyMMA(mffs, PARS, tempgrid, UID = "UID"),tempgrid) %>%
  mutate(Moose = "Yes") %>%
  bind_rows(
    cbind(RUNbyMMA(ffs, PARS, tempgrid, c(B = 100, J = 0.1, S = 100), UID = "UID"),tempgrid) %>%
      mutate(Moose = "No")
  )

mhsbplot = tibble(tempresults)  %>%
  select(UID, B, J, S, hsb, MMA, Moose) %>%
  pivot_longer(B:S) %>%
  pivot_wider(names_from = Moose) %>%
  mutate(EOM = 100*(Yes-No)/No) %>%
  left_join(
    tibble(name = c("B", "J", "S", "M"),
           name2 = c("Balsam fir", "Juv. Balsam fir", "Spruce", "Moose"))
  ) %>%
  ggplot(aes(x = hsb, y = EOM, group = MMA, color = MMA)) + geom_vline(xintercept = 1, linetype = 2) + geom_line() + facet_wrap(.~name2, scales = "free") + theme_classic() + scale_x_log10(name = "Moose habitat selection bias") + ylab("Effect of moose (%)") + scale_color_continuous(guide = "none")


# Death of juvenile fir caused by browsing (NOT INCLUDED IN MANUSCRIPT) ----

overkill <- function(okrate = 0, eMbase = unname(PARS["eM"]),cMJbase = unname(PARS["cMJ"])){
  
  cMJmod = (cMJbase + okrate)
  eMmod = eMbase*cMJbase/cMJmod
  
  stopifnot(all.equal(eMbase*cMJbase, eMmod*cMJmod))
  
  return(c(cMJ = cMJmod, eM = eMmod))
}

OKscenlist <- data.frame(t(sapply(seq(0, 10), overkill))) %>%
  mutate(OKscen = 1:11-1)

tempgrid = expand.grid(OKscen = 1:11,
                       MMA = 1:47) %>%
  inner_join(
    OKscenlist
  ) %>%
  inner_join(
    MMAabiotic2, by = "MMA"
  ) %>%
  select(-OKscen)


tempresults <- cbind(RUNbyMMA(mffs, PARS, tempgrid, UID = "UID"),tempgrid) %>%
  mutate(Moose = "Yes") %>%
  bind_rows(
    cbind(RUNbyMMA(ffs, PARS, tempgrid, c(B = 100, J = 0.1, S = 100), UID = "UID"),tempgrid) %>%
      mutate(Moose = "No")
  )


maeplot = tibble(tempresults)  %>%
  left_join(
    OKscenlist, by = c("cMJ", "eM")
  ) %>%
  select(UID, B, J, S, OKscen, MMA, Moose) %>%
  pivot_longer(B:S) %>%
  pivot_wider(names_from = Moose) %>%
  mutate(EOM = 100*(Yes-No)/No) %>%
  left_join(
    tibble(name = c("B", "J", "S", "M"),
           name2 = c("Balsam fir", "Juv. Balsam fir", "Spruce", "Moose"))
  ) %>%
  ggplot(aes(x = OKscen, y = EOM, group = MMA, color = MMA)) + geom_vline(xintercept = 1, linetype = 2) + geom_line() + facet_wrap(.~name2, scales = "free") + theme_classic() + scale_x_log10(name = "Excess J death") + ylab("Effect of moose (%)") + scale_color_continuous(guide = "none")


png("Plots/moose_alter_transition/mooseeffect_hsbae.png", width = 8, height = 5, units = "in", res = 600)
cowplot::plot_grid(
  mhsbplot,
  maeplot, ncol = 1, nrow = 2
)
dev.off()

png("Plots/moose_alter_transition/mooseeffect_hsb.png", width = 8, height = 4, units = "in", res = 600)
mhsbplot
dev.off()

# Eigenvalue moose ----

MMA %>%
  left_join(
    RUNbyMMA(mffs, PARS, MMAabiotic2, reteigenvalue = T) %>%
      pivot_longer(-MMA) %>%
      group_by(MMA) %>% 
      summarize(RT = 1/(-max(value)))
  ) %>%
  ggplot() + geom_sf(aes(fill = RT)) +
  scale_fill_viridis_c()

MMA %>%
  left_join(
    RUNbyMMA(mff, PARS, MMAabiotic2, startvec = c(B = 100, J = 0.1, M = 0.1), reteigenvalue = T) %>%
      pivot_longer(-MMA) %>%
      group_by(MMA) %>% 
      summarize(RT = 1/(-max(value)))
  ) %>%
  ggplot() + geom_sf(aes(fill = RT)) +
  scale_fill_viridis_c()


# Sense the flexibility of response ----

jacob_eqm <- function(FUNC, PARMS, yguess, rtnmax = T){
  eqm <- stode(y = yguess, func = FUNC, parms = PARMS)
  
  if(attributes(eqm)$steady){
    J <- jacobian.full(eqm$y, FUNC, parms = PARMS)
    if(rtnmax){
      max(
        Re(
          eigen(J)$values
        )
      )
    }else{
      return(J)
    }
  }else{
    return(0)
  }
}

solvemffs_rmax <- function(pars){
  
  rmax <- jacob_eqm(FUNC = mffs, PARMS=pars, yguess = c(B = 100, J = 0.1, M = 0.1, S = 100), rtnmax = T)
  
  return(c(rmax = rmax))
}

sense_function <- function(constpars, varpars){
  # Baseline parameters
  results = vector(mode = "list", dim(varpars)[1])
  
  for(i in 1:dim(varpars)[1]){
    
    PARS2 = constpars
    # Loop over each of the parameters in the grid and add it to the parameters vector
    for(iname in colnames(varpars)){
      PARS2[iname] = varpars[i,iname]
    }
    
    sensres <- sensFun(solvemffs_rmax, PARS2, sensvar = "rmax", senspar = names(PARS2))[-2]
    
    results[[i]] = tibble(names = names(sensres),
                          sens = as.numeric(as.matrix(sensres))) %>%
      filter(!is.na(sens) & names!= "x"& names!= "pb" & names != "MMA") %>%
      mutate(MMA = i)
  }
  
  return(do.call("rbind",results))
}

sensgraphic = sense_function(PARS, MMAabiotic2) %>%
  left_join(
    tibble(names = names(PARS),
           Category = c("Trees","Trees","Trees",
                        "Trees","Trees", "Moose", 
                        "Moose","Moose","Trees",
                        "Trees","Moose", "Moose",
                        "Moose","Moose","Trees", 
                        "Trees","Trees","Trees",
                        "Trees", "Trees", "Trees", "Trees"),
           names2 = c("a[BS]","a[JS]","a[SB]",
                      "r[B]","g[B]","hsb",
                      "e[M]","c[JM]","d[J]",
                      "r[S]","r[M]","K[M]", 
                      "beta", "h[M]","h[S]",
                      "h[B]","p[B]","p[S]",
                      "lambda","slope[lambda]", "K[S]", "K[B]")
    ), by = "names"
  ) %>%
  filter(names != "hsb" & names != "beta")




sensgraphic$names2 = factor(sensgraphic$names2, 
                           levels = sensgraphic %>%
                             group_by(names2) %>%
                             summarize(sensorder = -mean(abs(sens))) %>%
                             ungroup() %>% arrange(sensorder) %>% pull(names2))

sensgraphic  %>%
  ggplot(aes(x = names2, y = sens)) +
  geom_hline(yintercept = 0, linetype = 2) + 
  geom_boxplot(aes(fill = Category),outlier.shape = NA) + 
  geom_jitter(height = 0,width = 0.1,alpha = 0.3) +
  geom_text(aes(label = MMA), data = sensgraphic %>% filter(abs(sens) > 5), nudge_x = 0.5, size = 2) +
  theme_classic() + ylab("Sensitivity of return time") + xlab("Parameter") + scale_fill_viridis_d() + scale_x_discrete(labels = scales::parse_format())

# Simulate the return times by pool -----

# Baseline output:

sim_return <- function(focPAR = NA, PARStemp = PARS, mfact = 1.05, simtime = 1000, modvec = c(0.41, 1,1,0.17)){
  
  results = vector("list", dim(MMAabiotic2)[1])
  
  for(i in 1:length(results)){
    
    for(iname in colnames(MMAabiotic2)){
      PARStemp[iname] = MMAabiotic2[i,iname]
    }
    
    if(!is.na(focPAR)){
      PARStemp[focPAR] = PARStemp[focPAR]*mfact
      # print(PARStemp[focPAR])
    }
    
    sv = stode(y = c(B = 100, J = 0.1, M = 0.1, S = 100), func = mffs, parms = PARStemp)$y
    
    simout = as.data.frame(ode(y = sv*modvec, 1:simtime, mffs, PARStemp))
    
    fullsim = dim(simout)[1]
    
    results[[i]] = cbind(simout, MMA = i, simdim = fullsim) %>%
      pivot_longer(!time & !MMA & !simdim) %>%
      left_join(
        tibble(name = c("B", "J", "M", "S"),
               eqm = unname(sv)), by = "name"
      )
    
    rm(simout, fullsim)
  }
  
  out = do.call("rbind", results) %>%
    mutate(Parm = focPAR)
  
  return(out)
}

output = lapply(c(NA,names(PARS))[-21], sim_return)

# lambda_slope parameter doesn't work. Modification breaks the model.

gout = do.call("rbind",output)

gout$Parm = replace_na(gout$Parm, "Base")


glimit = gout %>%
  filter(time > 5) %>%
  mutate(diff = (value-eqm)/eqm) %>%
  filter(abs(diff) < 0.00001) %>%
  group_by(name, MMA, Parm) %>%
  slice_min(time)

# Plot the sensitivity to each parameter based on which pool takes the longest to return.

png("Plots/moose_alter_transition/sens_return_time_longestpool.png", width = 8, height = 5, units = "in", res = 600)
sensgraphic %>%
  left_join(
    glimit %>% filter(Parm == "Base") %>%
      ungroup() %>%
      select(time, MMA, name) %>%
      group_by(MMA) %>%
      slice_max(time) %>%
      ungroup() %>%
      select(MMA, name), by = "MMA"
  ) %>%
  ggplot(aes(x = name, y = sens)) + geom_boxplot() + geom_jitter(height = 0, width = 0.2, color = "grey") + facet_wrap(.~names2, scales = "free", labeller = "label_parsed") + xlab("Pool with longest simulated return") + ylab("Parameter sensitivity") + theme_classic() + geom_hline(yintercept = 0, linetype = 2)
dev.off()

# Plot the sensgraphic for the plots where spruce is the slowest to recover:

png("Plots/moose_alter_transition/sens_return_time.png", width = 8, height = 5, units = "in", res = 600)
sensgraphic %>%
  left_join(
    glimit %>% filter(Parm == "Base") %>%
      ungroup() %>%
      select(time, MMA, name) %>%
      group_by(MMA) %>%
      slice_max(time) %>%
      ungroup() %>%
      select(MMA, name), by = "MMA"
  ) %>%
  left_join(
    tibble(name = c("M", "S"),
           `Slowest recovery` = c("Moose", "Spruce")), by = "name"
  ) %>%
  ggplot(aes(x = names2, y = sens)) +
  geom_hline(yintercept = 0, linetype = 2) + 
  geom_boxplot(aes(fill = Category),outlier.shape = NA) + 
  geom_jitter(aes(color = `Slowest recovery`), height = 0,width = 0.1,alpha = 0.3) +
  geom_text(aes(label = MMA), data = sensgraphic %>% filter(abs(sens) > 5), nudge_x = 0.5, size = 2) +
  theme_classic() + ylab("Sensitivity of return time") + xlab("Parameter") + scale_fill_viridis_d(name = "Parameter Category") + scale_x_discrete(labels = scales::parse_format()) + scale_color_manual(values = c("red", "black"))
dev.off()

# Calculate the number of MMAs where moose or spruce are the slowest to recover
glimit %>% filter(Parm == "Base") %>%
  ungroup() %>%
  select(time, MMA, name) %>%
  group_by(MMA) %>%
  slice_max(time) %>%
  ungroup() %>%
  select(MMA, name) %>% group_by(name) %>% summarize(N = n())
