# Compare model versions in scripts 16, 17, and 18
# A: Robert W. Buchkowski
# D: Feb 15/2022

# Load in the packages:
if(!require("pacman")) install.packages("pacman")
p_load(tidyverse, ggpubr)

# Load in the saved data:

compdata = read_rds("Data/Data_for_model_version_compare/modeldata_basemodel.rds") %>%
  mutate(Model = "Base model") %>%
  bind_rows(
    read_rds("Data/Data_for_model_version_compare/modeldata_lambdamod.rds") %>%
      mutate(Model = "Lambda as function of moose biomass")
  ) %>%
  bind_rows(
    read_rds("Data/Data_for_model_version_compare/modeldata_fixedJ.rds") %>%
      mutate(Model = "Juv. fir set to empirical data")
  )


png(paste0("Plots/compare_model_means.png"), width = 8, height = 8, units = "in", res = 600)
compdata %>%
  ggplot(aes(x = value, y = mean, color = Model)) + geom_point(shape = 1) + theme_classic() + facet_wrap(.~name, scales = "free") + geom_abline(intercept = 0, slope = 1, linetype = 2, color = "grey20") + ylab("Empirical measurements") + xlab("Model predictions") +
  stat_regline_equation(label.x.npc = 0.1, label.y.npc = 0.95, aes(label = ..rr.label..)) +
  theme(legend.position = "top")
dev.off()

png(paste0("Plots/compare_model_EOM.png"), width = 6, height = 4, units = "in", res = 600)
compdata %>%
  filter(name %in% c("S", "B", "J")) %>%
  select(MMA, name, EOM, Model) %>%
  pivot_wider(names_from = Model, values_from = EOM) %>%
  ggplot(aes(x = `Base model`, y = `Lambda a function of moose biomass`)) + geom_point(shape = 1) + theme_classic() + facet_wrap(.~name, scales = "free") + geom_abline(intercept = 0, slope = 1, linetype = 2, color = "grey20")
dev.off()
