# Exploring the moose collar data
# Author: Robert W. Buchkowski
# Date: Sept 12/2022


# Read in collar data:

library(tidyverse)
library(sf)
library(ggpubr)

LF <- list.files("C:/Users/rober/Documents/Moose_Model_Original_Data/PaulSaunders_collardata/")

LF = paste0("C:/Users/rober/Documents/Moose_Model_Original_Data/PaulSaunders_collardata/",LF[grepl(".shp", LF)])

MMA2 <- read_sf("C:/Users/rober/Documents/Moose_Model_Original_Data/MMA/MMA.shp") %>%
  mutate(MMA = ifelse(MMA == "000", "006", MMA)) %>%
  group_by(MMA) %>% 
  summarize()

ggplot(MMA2) + geom_sf() + geom_sf_label(aes(label = MMA))

# Load in the forestry spatial database
forestryshape <- read_sf("Data/forest_inventory/inventory_spatial/forestry1.shp") %>%
  bind_rows(
    read_sf("Data/forest_inventory/inventory_spatial/forestry2.shp")
  ) %>%
  bind_rows(
    read_sf("Data/forest_inventory/inventory_spatial/forestry3.shp")
  )


# Load in the UID meanings:
forestryasp = read_rds("Data/forest_inventory/forestry.rds")

# Load in the moose range and create clipped shape:

moosepropfunction <- function(MID = 12, moose = LF, gsf = forestryshape, asf = forestryasp){
  mooseXXX <- read_sf(moose[[MID]])%>%
    st_transform(crs = st_crs(gsf))
  
  gsf2 = gsf %>%
    st_crop(
      mooseXXX  %>%
        st_bbox()
    )
  
  mooseXXX = mooseXXX %>%
    st_join(
      gsf2
    )
  
  
  mooseXXX %>%
    tibble() %>%
    select(UID) %>%
    group_by(UID) %>%
    summarize(N = n()) %>%
    left_join(
      asf, by = "UID"
    ) %>%
    group_by(FORESTTYPE) %>%
    summarize(N = sum(N)) %>%
    mutate(PropMoose = N/sum(N)) %>%
    full_join(
      gsf2 %>%
        mutate(Area = as.numeric(st_area(.))) %>%
        tibble() %>% select(-geometry) %>%
        left_join(
          asf, by = "UID"
        ) %>%
        group_by(FORESTTYPE) %>%
        summarize(Area = sum(Area))%>%
        mutate(PropArea = Area/sum(Area)), by = "FORESTTYPE"
    ) %>%
    mutate(Moose = MID)
 
}

# Moose 7 - 15 are winter moose:
moosepropfunction()

result <- lapply(7:15, moosepropfunction)


do.call("rbind",result) %>% write_rds("Data/moose_collar_prop_use.rds")

read_rds("Data/moose_collar_prop_use.rds") %>%
  filter(FORESTTYPE == 1) %>%
  ggplot(aes(x = PropArea, y = PropMoose)) + geom_point() + theme_classic()

# Convert proportion Moose and proportion area to a feeding estimate:

png(paste0("Plots/moose_collar_data.png"), width = 6, height = 6, units = "in", res = 600)
read_rds("Data/moose_collar_prop_use.rds") %>%
  filter(FORESTTYPE == 1) %>% # Choose the moose estimate in commercial forest
  mutate(PREF = PropMoose/PropArea) %>%
  ggplot(aes(x = PropArea, y = PropMoose)) + geom_point() + theme_classic() + 
  geom_abline(intercept = 0, slope = 1, linetype =2) + xlab("Proportion of area bounded by collar locations that is commercial forest") + ylab("Proportion of collar returns in commercial forest") + xlim(c(0,1))+ ylim(c(0,1))
dev.off()


summary(lm(PREF~PropArea, data = read_rds("Data/moose_collar_prop_use.rds") %>%
             filter(FORESTTYPE == 1) %>% # Choose the moose estimate in commerical forest
             mutate(PREF = PropMoose/PropArea)))

summary(lm(PropMoose~PropArea, data = read_rds("Data/moose_collar_prop_use.rds") %>%
             filter(FORESTTYPE == 1) %>% # Choose the moose estimate in commerical forest
             mutate(PREF = PropMoose/PropArea)))

summary(lm(PREF~1, data = read_rds("Data/moose_collar_prop_use.rds") %>%
             filter(FORESTTYPE == 1) %>% # Choose the moose estimate in commerical forest
             mutate(PREF = PropMoose/PropArea)))
