# Load in the elevation data and create a clean output file:
# Author: Robert W. Buchkowski
# Date: Jan. 15/2021

library(sf)
library(raster)
library(tidyverse)

tmp <- "C:/Users/rober/Documents/Moose_Model_Original_Data/Elevation"

LF <- list.files(tmp, recursive = T)

LF2 <- rep(NA, length(LF))
LF1 <- strsplit(LF, ".", fixed = T)

# New file paths
for(i in 1:length(LF)) LF2[i] = LF1[[i]][1]

# Unzip all elevation files
for(i in 1:length(LF)) unzip(paste0(tmp,"/",LF[i]), exdir = paste0(tmp,"/",LF2[i]))

# Load all the file paths
LF <- list.files(tmp, recursive = T)

# Get rid of all that aren't GeoTIFFs
LF = LF[grepl(".tif",LF, fixed = T)]
LFraster <- vector(mode = "list", length = length(LF))

# Load in all the raster sections
for(i in 1:length(LF)) LFraster[[i]] = raster(paste0(tmp, "/", LF[i]))

# Create a merged elevation raster
elevation <- sp::merge(LFraster[[1]], LFraster[[2]])

# The raster in 9 is empty, so we can remove it from the merge
seqtodo <- 3:length(LF)
seqtodo = seqtodo[seqtodo != 9]

for(i in seqtodo){
  elevation <- sp::merge(elevation, LFraster[[i]])
  print(i)
}

plot(elevation)

# Save the merged elevation raster as a .grd file.
writeRaster(elevation, "Data/Elevation/elevation.grd")
