# Wind Speed data extraction
# Author: Robert W. Buchkowski
# Date: March 25/2021

# Goal: Extract wind speed data for each forestry polygon

# Load in libraries
library(raster)
library(sf)
library(tidyverse)

LF = list.files("C:/Users/rober/OneDrive - The University of Western Ontario/SPI_Data_Backup/WindData_NRCan/canesm2_hist_swind_abs_1951-2005_ind/", recursive = T)

LF = LF[!grepl(".tif.aux", LF)]
LF = LF[!grepl(".tif.xml", LF)]

LF = paste0("C:/Users/rober/OneDrive - The University of Western Ontario/SPI_Data_Backup/WindData_NRCan/canesm2_hist_swind_abs_1951-2005_ind/", LF)

atest <- stack(LF[1:100])
test2 <- calc(test, sum)

test <- stack(LF[101:200])
test2b <- calc(test, sum)

test <- stack(LF[201:300])
test2c <- calc(test, sum)

test <- stack(LF[301:400])
test2cc <- calc(test, sum)

test <- stack(LF[401:500])
test2d <- calc(test, sum)

test <- stack(LF[501:600])
test2e <- calc(test, sum)

test <- stack(LF[601:660])
test2f <- calc(test, sum)

test2g <- test2 + test2b + test2c + test2cc + test2d + test2e + test2f

test_final <- test2g/600

writeRaster(test_final, "Data/windspeed/windspeed.tif")
plot(test_final)

# Load back in the average wind speed
windspeed <- raster("Data/windspeed/windspeed.tif")

for(i in 1:99){
  
  forestry2 <- read_sf(paste0("Data/FRI_subset/FRIsubset_",i,".shp"))
  
  emptyq <- st_dimension(forestry2)
  
  forestry2 <- forestry2[!is.na(emptyq),]
  
  # forestry2 <- st_make_valid(forestry2)
  
  forestry3 <- st_transform(forestry2, crs(windspeed))
  
  windspeedcrop <- crop(windspeed,extent(forestry3))
  
  t1 = Sys.time()
  forestry2 <- raster::extract(windspeedcrop, # Use the NL only temperature data
                               forestry3, # Use the FMD that is reprojected into the right coordinate system for the raster
                               method = "simple", 
                               small = T, # Allow the code to find a possible value outside the polygon if it doesn't cover a raster cell
                               fun = mean, # Take the mean of multiple overlapping raster cells
                               sp = T #add the extracted values to the forestry2 data frame
  ) 
  
  t2 = Sys.time()
  print(paste("Done",i, "in", t2-t1)) 
  
  write_rds(data.frame(forestry2), paste0("Data/FRI_windspeed_extraction/FRI_subset_", i, ".rds"))
}

# Compile the extracted wind speed data
tp_extract <- vector(mode = "list", length = 99)
for(i in 1:99) tp_extract[[i]] = read_rds(paste0("Data/FRI_windspeed_extraction/FRI_subset_", i, ".rds"))

tp_extract <- do.call("rbind", tp_extract)
write_rds(tp_extract, paste0("Data/FRI_windspeed_extraction/windspeed_extract.rds")) # Write this data

tp_extract <- read_rds(paste0("Data/FRI_windspeed_extraction/windspeed_extract.rds"))

# The 26837 NAs for wind speed
table(is.na(tp_extract$windspeed))

tp_extract = tp_extract[is.na(tp_extract$windspeed),]

# Pull the wind speed polygons without data
forestry <- vector(mode = "list", length = 99)
for(i in 1:99){
  forestry[[i]] <-
    read_sf(paste0("Data/FRI_subset/FRIsubset_",i,".shp")) %>%
    inner_join(tp_extract, by = "UID")
  print(i)
}

forestry <- do.call("rbind", forestry)

emptyq <- st_dimension(forestry)

forestry <- forestry[!is.na(emptyq),]

forestry3 <- st_transform(forestry, crs(windspeed))

windspeedcrop <- crop(windspeed,extent(forestry3))

forestry <- raster::extract(windspeedcrop, # Use the NL only temperature and precipitation data
                             st_centroid(forestry3[,1:2]), # Use the FMD that is reprojected into the right coordinate system for the raster
                             method = "simple", 
                             small = T, # Allow the code to find a possible value outside the polygon if it doesn't cover a raster cell
                             fun = mean, # Take the mean of multiple overlapping raster cells
                             na.rm = T,
                             buffer = 5300, # This buffer is chosen to have the minimum buffer that finds a non-zero raster value for each of the points in the data set
                             sp = T #add the extracted values to the forestry2 data frame
) # takes a longer time 

forestry = sf::st_as_sf(forestry, coords = c("x","y")) %>%
  tibble() %>%
  select(UID, windspeed)

read_rds(paste0("Data/FRI_windspeed_extraction/windspeed_extract.rds")) %>%
  filter(!(UID %in% forestry$UID)) %>%
  bind_rows(
    forestry
  ) %>%
  write_rds(paste0("Data/FRI_windspeed_extraction/windspeed_extract.rds"))

